/**************************************************************************
 ***          Programme du MCMT V   avec un PIC16C74                    ***
 ***                                                                    ***
 **************************************************************************
 ***          Programmeurs: Laurent Bernasconi                          ***
 ***          Version : 1.4-2001       PEC                              ***
 ***          Debut:                Fin:                                ***
 **************************************************************************/

#include <16c77.h>
//#include <16c74a.h>

#fuses  HS,NOWDT,NOPROTECT,NOBROWNOUT

#use delay(clock=20000000)
#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7)

#use fast_io(A)
#use fast_io(B)
#use fast_io(C)
#use fast_io(D)
#use fast_io(E)

//#BIT    CCP1IF = 0x0C.2
//#BIT    CCP2IF = 0x0D.0
//#BIT    TMR2IF = 0x0C.1
#BIT    TOIF = 0x0B.2
//#BIT    RP0 = 0x03.5
//#BIT    RP1 = 0x03.6

//#BIT    RCIF = 0x0C.5
#BIT    TRMF = 0x98.1

#BIT     BRGH = 0x98.2
#byte   SPBRG = 0x99

long TIMER_1;
#byte   TIMER_1 = 0x0E

//#byte   OPTION = 0x81
#byte PORTA = 5
#byte PORTB = 6
#byte PORTC = 7
#byte PORTD = 8
#byte PORTE = 9

/****************************************************************
*               Definition de la version                        *
****************************************************************/

#define version         20


/****************************************************************
*                  Definition des Output                        *
****************************************************************/

/*
 *      Bits de control des moteurs *      RA0..3 pour la phase A
 *      RD0..3 pour la phase B
 *
 */

#define PHASE          	PIN_A0  // Selection de la phase
#define WR_N          	PIN_A1  // MAX505
#define LDAC_N          PIN_A2  // MAX505
#define DIR_PHASEA 	PIN_A3  // Direction du courant dans la phase
#define DIR_PHASEB  	PIN_A5

#define COURANT_M3      PIN_C2  // Amplitude du courant dans le moteur
#define COURANT_M4      PIN_C5

/*
 *      Bits de control des LED
 *
 */

#define LED_A_Moins 	PIN_C0
#define LED_A_Plus  	PIN_C1

/****************************************************************
*                  Definition des Input                         *
****************************************************************/

/*
 *      Bits des signaux d'entrees
 *
 */
#define ST4_Plus  	PIN_B0
#define ST4_Moins  	PIN_B1
#define RAQ_Plus  	PIN_B2
#define RAQ_Moins  	PIN_B3
#define VIT1   		PIN_B4
#define VIT2   		PIN_B5
#define PEC1   		PIN_B6
#define PEC2   		PIN_B7
#define PEC3   		PIN_A4

/****************************************************************
*     Adresses des data dans la memoire EEPROM                  *
****************************************************************/
#define VM_Guidage   		0
#define VM_Guidage_LSB   	2
#define VM_Corec_Plus    	3
#define VM_Corec_Moins    	5
#define VM_lent        		7
#define VM_rapide      		9
#define VM_acce   		11
#define Dir_Guidage  		12
#define Dir_Corec_Plus  	13
#define Dir_Corec_Moins  	14
#define Dir_Lent_plus  		15
#define Dir_lent_Moins  	16
#define Dir_Rapide_Plus  	17
#define Dir_rapide_moins 	18
#define Resol_Guidage  		19
#define Resol_Corec_Plus 	20
#define Resol_Corec_Moins 	21
#define Resol_Lent  		22
#define Resol_rapide  		23
#define Courant_Guidage  	24
#define Courant_Lent  		25
#define Courant_Rapide  	26
#define Sens_Raq_Led  		27

#define Corect_Courant_1 	28
#define Corect_Courant_2 	29
#define Corect_Courant_3 	30


/*
 * Vecteur determinant la sequence des micropas
 * pour une resolution de 128 micropas par pas
 * selon une courbe sinusoidale
 *
 */

byte const T_STEP[256] ={
0,3,6,9,13,16,19,22,25,28,31,34,37,41,44,47,50,53,56,59,62,
65,68,71,74,77,80,83,86,89,92,95,98,100,103,106,109,112,115,
117,120,123,126,128,131,134,136,139,142,144,147,149,152,154,
157,159,162,164,167,169,171,174,176,178,180,183,185,187,189,
191,193,195,197,199,201,203,205,207,208,210,212,214,215,217,
219,220,222,223,225,226,228,229,231,232,233,234,236,237,238,
239,240,241,242,243,244,245,246,247,247,248,249,249,250,251,
251,252,252,253,253,253,254,254,254,255,255,255,255,255,255,
255,255,255,255,255,254,254,254,253,253,253,252,252,251,251,
250,249,249,248,247,247,246,245,244,243,242,241,240,239,238,
237,236,234,233,232,231,229,228,226,225,223,222,220,219,217,
215,214,212,210,208,207,205,203,201,199,197,195,193,191,189,
187,185,183,180,178,176,174,171,169,167,164,162,159,157,154,
152,149,147,144,142,139,136,134,131,128,126,123,120,117,115,
112,109,106,103,100,98,95,92,89,86,83,80,77,74,71,68,65,62,
59,56,53,50,47,44,41,37,34,31,28,25,22,19,16,13,9,6,3
};


/***************************************************************************
 ******         Definition des variables        ****************************
 ***************************************************************************/

/*
 * Variables globales
 *
 */
//int  Corection_Point_Zero;
//int  Future_Corection_Point_Zero;

//int saved_w,saved_status,saved_fsr,saved_pclath;
//long  temp, pente;

short   Posi;                   // Drapeau pour le positionnement en pas
short 	valide_Posi;

long    NUM_POSI_L;
long    NUM_POSI_H;

long    NUM_COMP_L;
long    NUM_COMP_H;
                                // short pour le controle de la direction du moteur
short   Flag_DIR_PHASEA;
short   Flag_DIR_PHASEB;

int 	V_M_LSB_Compteur; // poid fort du compteur 24 bits
int     V_M_LSB;
long  	V_M;               // Vitesse du Moteur 1 partie 16 bits
long    Sauve_VM;
int   	Resolution_M;
short 	Dir_M;
short 	Ancien_Dir_M;
int  	Niveau_Courant_M;
int 	Accel_M;
int  	Resol_Min;

int 	New_V_M_LSB;
long  	New_V_M;               // Liste des futures valeurs moteur.
int    	New_Resolution_M;
short 	New_Dir_M;
int  	New_Niveau_Courant_M;

long 	Vitesse_V_M_But;
int 	Future_V_M_LSB;
long  	Future_V_M;               // Liste des futures valeurs moteur.
int    	Future_Resolution_M;
short 	Future_Dir_M;
int  	Future_Niveau_Courant_M;

int     Etat_Controle;          //Registre d'etat du controle
                                //Bit0  Rotation        1 -> Oui
                                //Bit1  Dir             1 -> Sens Horaire
                                //Bit2  Rampe+          1 -> Oui
                                //Bit3  Rampe-          1 -> Oui
                                //Bit4  Vitesse         Guidage = 0, Correct+ = 1, Correct- = 2,
                                //Bit5  Vitesse         Lent = 3, Rapide = 4
                                //Bit6  Vitesse
                                //Bit7  En mouvement    1 -> Oui (Futur!!!)

int 	NUM_STEP_M_A;           // Numero du microstep sur une base de 256 pour 180 degree e
int  	NUM_STEP_M_B;

char  	Buf_serial[7];
char 	Buf_Copie[6];
char 	Buf_Raq[2];
int  	Buf_count;
int	Sincro;
short   Serial_rec;             // Drapeau de reception d'une commande serie
short 	Raq_rec;
short 	Sens;
short 	Ordre_Vitesse;
short  	Demande_Moteur;
short   Rampe_Plus;
short 	Rampe_Moins;
short   Change;
short 	Inverse_Sens;
short  	Guidage;
short   Fixe_Ordre_Vitesse_0;
//short	Siderale;

int     Num_Carte;
int 	Clavier;
long 	Vit_Ram;
int 	Vit_Ram_LSB;
short 	Selec_Vit_Ram;
int 	Resol_Ram;
short 	Dir_Ram;

short	Pos_Pec1;
long	Pec1_NUM_POSI_L;
long	Pec1_NUM_POSI_H;
int	Clavier_Pec;

//int  Etat_Raquette;



/*************************************************************************************/
/********************************      Fonctions      ********************************/
/*************************************************************************************/

#define EEPROM_SDA  PIN_C4
#define EEPROM_SCL  PIN_C3

//#include <2465.C>
//#include <2401.C>
#include <2408.C>



/****************************************************************
*       Function Name:  Init_PARAMETRE                          *
*       Return Value:   long                                    *
*       Parametres:     adresse de la variable                  *
*       Description:    initialisation 16 bits ds eeprom        *
****************************************************************/
long Init_PARAMETRE(long int address)
{
        char * var_pointeur;
        long RESULT;

        var_pointeur = (char *)&RESULT;
        var_pointeur[0] = READ_EXT_EEPROM( address );
        var_pointeur[1] = READ_EXT_EEPROM( address+1 );

        return(RESULT);
}

/****************************************************************
*       Function Name:  reglage_Courant                         *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:                *
****************************************************************/
void Reglage_Courant()
{
        if (Niveau_Courant_M == 1)
 	{
         	OUTPUT_HIGH(COURANT_M3);// 46.7%
           	OUTPUT_LOW(COURANT_M4);
	}

        else if (Niveau_Courant_M == 2)
 	{
         	OUTPUT_LOW(COURANT_M3); // 73.3%
         	OUTPUT_HIGH(COURANT_M4);
 	}
        else if (Niveau_Courant_M == 3)
 	{
         	OUTPUT_HIGH(COURANT_M3);// 100%
                OUTPUT_HIGH(COURANT_M4);
 	}
        else
 	{
         	OUTPUT_LOW(COURANT_M3); // 20%  Au cas ou la valeur serait differente de 0, 1, 2 ou 3.
                OUTPUT_LOW(COURANT_M4);
 	}
}



/****************************************************************
*       Function Name:  step_moteur                             *
*       Return Value:   void                                    *
*       Parametres:     Numero selon la table des pas et moteur *
*       Description:    Generation des pas d'un moteur          *
****************************************************************/

void step_moteur(byte NUM_STEP_A, byte NUM_STEP_B)
{
        OUTPUT_LOW(PHASE);    // ecriture pour phase A
 	PORTD = T_STEP[NUM_STEP_A];
        OUTPUT_LOW(WR_N);
        OUTPUT_HIGH(WR_N);

        OUTPUT_HIGH(PHASE);                      // ecriture pour phase B
 	PORTD = T_STEP[NUM_STEP_B];
        OUTPUT_LOW(WR_N);
        OUTPUT_HIGH(WR_N);

        OUTPUT_LOW(LDAC_N);   // Validation du DAC
        OUTPUT_HIGH(LDAC_N);

 	if (resolution_M !=0)           // a ne pas faire si moteur a l'arret!!!
 	{
         	if(NUM_STEP_A ==0) // positionne ligne DirA.
         	{
   			//if (guidage)
    			//	delay_us(50);
                	if(Flag_DIR_PHASEA)
                 	{
                        	OUTPUT_LOW(DIR_PHASEA);
           			Flag_DIR_PHASEA = 0;
                 	}
                  	else
                 	{
                         	OUTPUT_HIGH(DIR_PHASEA);
                         	Flag_DIR_PHASEA = 1;
                 	}
         	}

         	else if (NUM_STEP_B ==0) // positionne ligne DirB.
         	{
   			//if (guidage)
    			//	delay_us(50);
                	if(Flag_DIR_PHASEB)
                	{
                      		OUTPUT_LOW(DIR_PHASEB);
                        	Flag_DIR_PHASEB = 0;
                 	}
                 	else
                 	{
                        	OUTPUT_HIGH(DIR_PHASEB);
                        	Flag_DIR_PHASEB = 1;
                 	}
         	}
 	}

        if (demande_Moteur)   // debut de test si modification vitesse demande.
        {
        	if (!Ordre_Vitesse)  // utilise pour faire le choix entre le mode guidage et pointage
                {
                 	Ancien_Dir_M= Dir_M;
          		V_M_LSB = New_V_M_LSB;
                 	V_M = New_V_M;     /* Moteur 1 guidage */
                 	Resolution_M = New_Resolution_M;
                 	if (resolution_M !=0)  // ne modifie le sens que si la resolution est != de 0
                  		Dir_M = New_Dir_M;
                 	if (Dir_M != Ancien_Dir_M) // ne modifie que si le sens change!
                 	{
                  		if (NUM_STEP_M_A==0)
                  		{
                   			if (Flag_DIR_PHASEA==1)
                   			{
                    				OUTPUT_LOW(DIR_PHASEA);
                    				Flag_DIR_PHASEA = 0;
                  	 		}
                   			else
                   			{
                    				OUTPUT_HIGH(DIR_PHASEA);
                    				Flag_DIR_PHASEA = 1;
                   			}
                  		}
                  		else if (NUM_STEP_M_B==0) // ne modifie que si le sens change!
                  		{
                   			if (Flag_DIR_PHASEB==1)
                   			{
                    				OUTPUT_LOW(DIR_PHASEB);
                    				Flag_DIR_PHASEB = 0;
                   			}
                   			else
                   			{
                    				OUTPUT_HIGH(DIR_PHASEB);
                    				Flag_DIR_PHASEB = 1;
                   			}
                  		}
                 	}
                 	Niveau_Courant_M = New_Niveau_Courant_M;
                 	Reglage_Courant();
          	 	Guidage = false;
                 	demande_Moteur=false;
                 	Ordre_Vitesse=false;
    		}
        	else
        	{
          		if (Resolution_M==0)  // permet a la motorisation de reprendre apres arret
                  		resolution_M=Resol_Min;  // il est mieux que la valeur soit egale a la resolution utilise pour le mode de guidage.
                	if (NUM_STEP_M_A==0)  // attend que le moteur soit a un endroit bien precis
                	{
                  		Ancien_Dir_M = Dir_M;
           			V_M_LSB = New_V_M_LSB;
                  		V_M = New_V_M;     /* Moteur 1 guidage */
                  		Resolution_M = New_Resolution_M;
                  		if (resolution_M!=0) // ne change le sens que si le moteur doit se deplace!=> Resolution <> de 0.
                   			Dir_M = New_Dir_M;
                  		if (Dir_M != Ancien_Dir_M) // ne change le sens que si utile.
                  		{
                   			if ( Flag_Dir_PhaseA==1)
                   			{
                    				OUTPUT_LOW(DIR_PHASEA);
                    				Flag_DIR_PHASEA = 0;
                   			}
                   			else
                   			{
                    				OUTPUT_HIGH(DIR_PHASEA);
                    				Flag_DIR_PHASEA = 1;
                   			}
                  		}
                  		Niveau_Courant_M = New_Niveau_Courant_M;
                  		Reglage_Courant();
           			if (Fixe_Ordre_Vitesse_0)
           			{
            				Ordre_Vitesse = 0;
            				Fixe_Ordre_Vitesse_0= false;
            				Guidage = False;
            			}
                  		demande_Moteur=false;
           		}
     		}
	}
}

/****************************************************************
*       Function Name:  Raquette                                *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:                *
****************************************************************/
void Raquette()
{

 	if (!Bit_Test(Clavier,0)) //ST4+ // Priorite ST4  // Fait comme si reception RS232
         	Buf_Raq[1] = 0x44; //D

        else if (!Bit_Test(Clavier,1)) //ST4-
         	Buf_Raq[1] = 0x51; //Q

        else if (!Bit_Test(Clavier,2)) // A+
 	{
         	if (Bit_Test(Clavier,4)) // Vit1 //Guidage
  		{
   			if (!sens)
           			Buf_Raq[1] = 0x44;  //D
   			else
    				Buf_Raq[1] = 0x51;  //Q
         	}
  		else
  		{
           		if (Bit_Test(Clavier,5)) //Vit2 // lent
   			{
    				if (!sens)
            				Buf_Raq[1] = 0x47; //G
    				else
   		  			Buf_Raq[1] = 0x46; //F
   			}
         	 	else     // Rapide
   				{
    				if (!sens)
            				Buf_Raq[1] = 0x58; //X
    				else
     					Buf_Raq[1] = 0x57; //W
   				}
  		}
 	}
        else if (!Bit_Test(Clavier,3)) //A-
 	{
         	if (Bit_Test(Clavier,4)) //Vit1 //Guidage
  		{
   			if (!sens)
           			Buf_Raq[1] = 0x51;  //Q
   		else
    			Buf_Raq[1] = 0x44;  //D
  		}
         	else
  		{
           		if (Bit_Test(Clavier,5)) //Vit2 // lent
   			{
    				if (!sens)
            				Buf_Raq[1] = 0x46; //F
    				else
     					Buf_Raq[1] = 0x47; //G
   			}
          		else     // Rapide
   			{
    				if (!sens)
            				Buf_Raq[1] = 0x57; //W
    				else
     					Buf_Raq[1] = 0x58; //X
   			}
  		}
 	}
        else   // Etat normal
         	Buf_Raq[1] = 0x53; //S

        Raq_rec =1;
}

/****************************************************************
*       Function Name:  Etat_Moteur                             *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:              *
****************************************************************/
void Etat_Moteur()
{

 	long    Resolution_M_Long;
 	long    Future_Resolution_M_Long;
 	long    Vitesse_M;
 	long    Future_Vitesse_M;

        if (!Ordre_Vitesse)
        {
  		//printf ("1");
         	V_M_LSB   = Future_V_M_LSB;
                New_V_M   = Future_V_M;
                New_resolution_M  = Future_resolution_M;
                New_Dir_M   = Future_Dir_M;
                New_Niveau_Courant_M  = Future_Niveau_Courant_M;
                demande_Moteur=true;
        }
        else
        {
  		//printf ("2");
         	if (resolution_M == 0)    // Attention a ces testes en fonction du mode modeur alpha ou delta!!!
    			Resolution_M_Long = (long)Resol_Min;
         	else
          		Resolution_M_Long = (long)Resolution_M;

         	if (Future_resolution_M == 0)
   			Future_Resolution_M_Long = (long)Resol_Min;
         	else
          		Future_Resolution_M_Long = (long)Future_resolution_M;

  		Vitesse_M = V_M / Resolution_M_Long;
         	Future_Vitesse_M = Future_V_M / Future_Resolution_M_Long;



         	if ((Future_Dir_M == Dir_M) || (Resolution_M == 0))  // meme sens
        	{
  			//printf ("3");
                 	if ( Vitesse_M == Future_Vitesse_M)
                 	{
    				//printf ("4");
           			V_M_LSB   = Future_V_M_LSB;
                  		New_V_M   = Future_V_M;
                  		New_resolution_M = Future_resolution_M;
           			New_Dir_M   = Future_Dir_M;
                  		New_Niveau_Courant_M  = Future_Niveau_Courant_M;
                         	demande_Moteur=true;
                 	}

                 	else if (Vitesse_M < Future_Vitesse_M)     // Diminution de la vitesse
                 	{
    				//printf ("5");
           			New_V_M   = Future_V_M;
                         	rampe_moins=true;
                 	}
                 	else      // augmentation de la vitesse
                 	{
    				//printf ("6");
           			Vitesse_V_M_But  = Future_V_M;
           			New_V_M   = V_M;
           			New_resolution_M = Future_resolution_M;
    				New_Dir_M   = Future_Dir_M;
                  		New_Niveau_Courant_M  = Future_Niveau_Courant_M;
                         	demande_Moteur=true;
                         	rampe_plus=true;
                 	}
            	}
                else   // changement de sens
         	{
   			//printf ("7");
          		New_V_M    = Init_PARAMETRE(VM_Guidage); // Vitesse de Guidage
                        rampe_moins=true;  // Enclanche diminution de la vitesse
          		Inverse_Sens=true;
                }
        }
        Change=false;
}

/****************************************************************
*       Function Name:  traite_serial                           *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:    Traitement des commandes du port serie  *
****************************************************************/
void traite_serial()
{
        char achar;
        int i;
        long int address;

 	if (serial_rec)
 	{
  		Buf_Copie[1]=Buf_Serial[1];
  		Buf_Copie[2]=Buf_Serial[2];
  		Buf_Copie[3]=Buf_Serial[3];
  		Buf_Copie[4]=Buf_Serial[4];
                Buf_Copie[5]=Buf_Serial[5];
  		Buf_serial[0] = '\0';
         	Buf_serial[1] = 0;
         	Buf_count = 0;
  		serial_rec =0;
 	}

 	else
 	{
  		Buf_Copie[1]=Buf_raq[1];
  		Raq_rec =0;
 	}

	if(	(Buf_Copie[1] == Num_Carte) &&
		(Buf_Copie[2] == Num_Carte) &&
		(Buf_Copie[3] == Num_Carte) &&
		(Buf_Copie[4] == Num_Carte)	)
        {
                disable_interrupts(global);
                while(true);
        }

        if(	(Buf_Copie[1] == 0x50) ||
		(Buf_Copie[1] == 0x70) ||
		(Buf_Copie[1] == 0x71))  //P ou p   POSITIONNEMENT
        {        // Grand P = Vitesse rapide
                #asm       //petit p = vitesse lente
                        MOVF   Buf_Copie[5],W    // petit q = vitesse de correction
                        MOVWF  NUM_COMP_L
                        MOVF   Buf_Copie[4],W
                        MOVWF  (&NUM_COMP_L+1)
                        MOVF   Buf_Copie[3],W
                        MOVWF  (&NUM_COMP_L+2)
                        MOVF   Buf_Copie[2],W
                        ANDLW  0x07
                        MOVWF  (&NUM_COMP_L+3)
                #endasm
                if(Buf_Copie[1] == 0x50)
     		{
                 	if(Bit_Test(Buf_Copie[2],7))
                         	Buf_Copie[1] = 0x57;
                 	else
                         	Buf_Copie[1] = 0x58;
   			Valide_Posi=false;
   			Posi = true;
   		}
  		else if(Buf_Copie[1] == 0x70)
   		{
                 	if(Bit_Test(Buf_Copie[2],7))
                         	Buf_Copie[1] = 0x46;
                 	else
                         	Buf_Copie[1] = 0x47;
   			Valide_Posi=false;
   			Posi = true;
   		}
  		else
   		{
                 	if(Bit_Test(Buf_Copie[2],7))
                         	Buf_Copie[1] = 0x51;
                 	else
                         	Buf_Copie[1] = 0x44;
   			Valide_Posi=true;
   			Posi = false;
   		}

      	}

 	if (	(Buf_Copie[1] == 0x53) ||
		(Buf_Copie[1] == 0x44) ||
		(Buf_Copie[1] == 0x51) ||
		(Buf_Copie[1] == 0x47) ||
		(Buf_Copie[1] == 0x46) ||
		(Buf_Copie[1] == 0x58) ||
 		(Buf_Copie[1] == 0x57)  )
        {
         	demande_Moteur=false;
                rampe_plus=false;
         	rampe_moins=false;
         	Inverse_Sens=false;
         	Fixe_Ordre_Vitesse_0=false;
        }


        if (Buf_Copie[1] == 0x43) //C   Lecture du compteur de micro-pas 32 bits
        {                               // MSB...LSB
		OUTPUT_HIGH(PEC3);
                printf("%c%c%c%c",NUM_POSI_H >> 8,NUM_POSI_H,NUM_POSI_L >> 8,NUM_POSI_L);
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
        }

        else if(Buf_Copie[1] == 0x53)  //S   25600 micro-pas
        {
  		if (Selec_Vit_Ram)
  		{
   			Future_V_M_LSB=  Vit_Ram_LSB;
          		Future_V_M =  Vit_Ram;
   			Future_resolution_M =  Resol_Ram;
          		Future_Dir_M=   Dir_Ram;
  		}
  		else
  		{
          		Future_V_M_LSB=   READ_EXT_EEPROM(VM_Guidage_LSB);
          		Future_V_M =    Init_PARAMETRE(VM_Guidage);
   			Future_resolution_M =   READ_EXT_EEPROM(Resol_Guidage);
          		Future_Dir_M=    READ_EXT_EEPROM(Dir_Guidage);
  		}

         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Guidage);
  		OUTPUT_LOW(Led_A_Plus);
  		OUTPUT_LOW(Led_A_Moins);
                Etat_Controle = 0x00;
                Change = true;
        	Guidage = true;
        }

        else if(Buf_Copie[1] == 0x44)  //D
        {
         	Future_V_M =    Init_PARAMETRE(VM_Corec_Plus);
         	Future_resolution_M =   READ_EXT_EEPROM(Resol_Corec_Plus);
         	Future_Dir_M=    READ_EXT_EEPROM(Dir_Corec_Plus);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Guidage);
  		If (!Sens)
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
         	Etat_Controle = 0x10;
                Change = true;
         	Guidage = true;
        }

        else if(Buf_Copie[1] == 0x51)  //Q
        {
         	Future_V_M =    Init_PARAMETRE(VM_Corec_Moins);
         	Future_resolution_M =   READ_EXT_EEPROM(Resol_Corec_Moins);
         	Future_Dir_M=    READ_EXT_EEPROM(Dir_Corec_Moins);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Guidage);
  		If (!Sens)
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
         	Etat_Controle = 0x20;
                Change = true;
         	Guidage = true;
        }

        else if (Buf_Copie[1] == 0x47)   //G  16*
        {
         	Future_V_M =    Init_PARAMETRE(VM_Lent);
         	Future_resolution_M =   READ_EXT_EEPROM(Resol_Lent);
         	Future_Dir_M=    READ_EXT_EEPROM(Dir_Lent_Plus);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Lent);
  		If (!Sens)
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
         	guidage = false;
         	Ordre_Vitesse=1;
         	Etat_Controle = 0x30;
                Change = true;
        }

        else if (Buf_Copie[1] == 0x46)  //F  16*
        {
         	Future_V_M =    Init_PARAMETRE(VM_Lent);    // c'est ici que la EEPROM prepare le changement de vitesse
         	Future_resolution_M =   READ_EXT_EEPROM(Resol_Lent);
         	Future_Dir_M=    READ_EXT_EEPROM(Dir_Lent_Moins);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Lent);
  		If (!Sens)
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
         	guidage = false;
         	Ordre_Vitesse=1;
         	Etat_Controle = 0x40;
                Change = true;
        }

        else if (Buf_Copie[1] == 0x58) //X  700*!!!!!  80S/micro-pas 3200 micro-pas/tour.
        {
         	Future_V_M =    Init_PARAMETRE(VM_Rapide);
         	Future_Resolution_M =   READ_EXT_EEPROM(Resol_Rapide);
         	Future_Dir_M =    READ_EXT_EEPROM(Dir_Rapide_Plus);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Rapide);
  		If (!Sens)
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
         	guidage = false;
         	Ordre_Vitesse=1;
         	Etat_Controle = 0x50;
                Change = true;
        }

        else if (Buf_Copie[1] == 0x57)  //W   700*!!!!
        {
         	Future_V_M =    Init_PARAMETRE(VM_Rapide);
         	Future_resolution_M =   READ_EXT_EEPROM(Resol_Rapide);
         	Future_Dir_M=    READ_EXT_EEPROM(Dir_Rapide_Moins);
         	Future_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Rapide);
  		If (!Sens)
  		{
   			OUTPUT_LOW(Led_A_Plus);
   			OUTPUT_HIGH(Led_A_Moins);
  		}
  		else
  		{
   			OUTPUT_HIGH(Led_A_Plus);
   			OUTPUT_LOW(Led_A_Moins);
  		}
         	guidage = false;
         	Ordre_Vitesse=1;
         	Etat_Controle = 0x60;
                Change = true;
        }

        else if (Buf_Copie[1] == 0x45) //E   Registre d'etat du controle
        {
                Etat_Controle &= 0x70;
                if(Resolution_M > 0)
                        Etat_Controle |= 0x01;  //En Rotation
                if(Dir_M)
                        Etat_Controle |= 0x02;  //Dir
                if(Rampe_Plus)
                        Etat_Controle |= 0x04;  //Rampe+
                if(Rampe_Moins)
                        Etat_Controle |= 0x08;  //Rampe-
                if(valide_posi)
                        Etat_Controle |= 0x80;  //En Positionnement
		OUTPUT_HIGH(PEC3);
                printf("%c%c",Num_Carte,Etat_Controle);
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
        }

        else if (Buf_Copie[1] == 0x4B) //K   Lecture EEPROM
        {
		OUTPUT_HIGH(PEC3);
         	for (i=0;i<31;i++)
                {
                   	achar=READ_EXT_EEPROM(VM_Guidage+i);
          		printf("%c",achar);
         	}
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
        }

        else if (Buf_Copie[1] == 0x4C) //L  Ecriture EEPROM
        {
         	address = VM_Guidage+Buf_Copie[2];
         	WRITE_EXT_EEPROM( address, Buf_Copie[3] );
                if(address < 11 && address != 2)
                        WRITE_EXT_EEPROM( address+1, Buf_Copie[4] );
  		else if (address==12)
   			Accel_M=READ_EXT_EEPROM(VM_acce);
  		else if ((address==19) || (address==20) || (address==21))
  		{
   			if (READ_EXT_EEPROM(Resol_Guidage)!=0)
    				resol_Min = READ_EXT_EEPROM(Resol_Guidage);
   			else if (READ_EXT_EEPROM(Resol_Corec_Plus)!=0)
    				resol_Min = READ_EXT_EEPROM(Resol_Corec_Plus);
   			else resol_Min = READ_EXT_EEPROM(Resol_Corec_Moins);
  		}
  		else if (address==27)
   			Sens=READ_EXT_EEPROM(Sens_Raq_Led);
        }

 	else if (Buf_Copie[1] == 0x56) //V  Lecture Version
 	{
  		achar=version;
		OUTPUT_HIGH(PEC3);
  		printf ("%c",achar);
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
 	}

 	else if (Buf_Copie[1] == 0x52) //R  Ecriture Valeur Vit_Ram, Vit_Ram_LSB, Resol_Ram et Dir_Ram
 	{
  		#asm
   			MOVF   Buf_Copie[4],W
                        MOVWF  Vit_Ram_LSB
                        MOVF   Buf_Copie[3],W
                        MOVWF  (&Vit_Ram)
                        MOVF   Buf_Copie[2],W
                        MOVWF  (&Vit_Ram+1)
                #endasm

  		Resol_Ram = Buf_Copie[5] & 0x7F;

  		Buf_Copie[5] = Buf_Copie[5] & 0x80;
  		if (Buf_Copie[5] == 0)
   			Dir_Ram   = 0;
  		else
   			Dir_Ram =1;
 	}

 	else if (Buf_Copie[1] == 0x72) //r  Lecture Valeur Vit_Ram, Vit_Ram_LSB, Resol_Ram et Dir_Ram
 	{
		OUTPUT_HIGH(PEC3);
  		printf("%c%c%c%c%c",Vit_Ram >> 8,Vit_Ram,Vit_Ram_LSB,Resol_Ram,Dir_Ram);
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
 	}

  	else if (Buf_Copie[1] == 0x73) //s  Selection Vit_Ram / Vit_EEPROM
 	{
 	 	Selec_Vit_Ram = Buf_Copie[2];
 	}

	else if (Buf_Copie[1] == 0x6A) //j   Lecture du compteur de micro-pas 32 bits PEC!!!
        {                               // MSB...LSB
		OUTPUT_HIGH(PEC3);
                printf("%c%c%c%c",Pec1_NUM_POSI_H >> 8,Pec1_NUM_POSI_H,Pec1_NUM_POSI_L >> 8,Pec1_NUM_POSI_L);
		while(!TRMF);
           	OUTPUT_LOW(PEC3);
        }
}

/**************************************************************************
 * Routine traitant les interruptions
 *
 * Timer0 RTTC overflow a toutes les 6528 uS
 * servant pour les rampes des moteurs
 * et la lecture du clavier
 *
 * CCP1 en comparaison avec Timer1
 * pour la generation des pas du moteur
 *
 *
 * Lecture du USART
 ***************************************************************************/
#int_CCP1
void IntCCP1()
{

	long  temp;

	temp = TIMER_1;

         //CCP_1 = V_M + temp;
  	CCP_1 = V_M;

        if(Valide_Posi)
        {
             	#asm                            // Soustraction en 32 bits
                       	MOVLW  1                //NUM_COMP -= 1;
                        SUBWF  NUM_COMP_L,F
                        BTFSC  03,0
                        GOTO   SUITE
                        SUBWF  (&NUM_COMP_L+1),F
                        BTFSC  03,0
                        GOTO   SUITE
                        SUBWF  (&NUM_COMP_L+2),F
                        BTFSS  03,0
                        DECF   (&NUM_COMP_L+3),F
                SUITE:
             	#endasm

                if((NUM_COMP_L == 0) && (NUM_COMP_H == 0))
                {
                        Buf_Raq[1] = 0x53; //S
                        Raq_Rec =1;
                        valide_Posi = false;
                }
     	}
        else
        {
               	if(++V_M_LSB_Compteur <= V_M_LSB)
                         ++CCP_1;

                if(V_M_LSB_Compteur == 10)
                         V_M_LSB_Compteur = 0;
        }

        if(Dir_M)
        {
                NUM_STEP_M_A+=Resolution_M;
                NUM_STEP_M_B+=Resolution_M;
                #asm                              // Addition en 32 bits
                 	MOVF   Resolution_M,W     //NUM_POSI_L += Resolution_M;
                        ADDWF  NUM_POSI_L,F
                        BTFSS  03,0
                        GOTO   SUITE1
                        MOVLW  1
                        ADDWF  (&NUM_POSI_L+1),F
                        BTFSS  03,0
                        GOTO   SUITE1
                        ADDWF  (&NUM_POSI_L+2),F
                        BTFSC  03,0
                        INCF   (&NUM_POSI_L+3),F
                SUITE1:
                #endasm

		Clavier_Pec=PORTB;
		if (!Pos_Pec1)
		{
			if (Bit_Test(Clavier_Pec,6))
			{
				Pos_Pec1 = true;
				//OUTPUT_HIGH(Led_A_Plus);
				Pec1_NUM_POSI_L = NUM_POSI_L;
				Pec1_NUM_POSI_H = NUM_POSI_H;
			}
		}
		else
		{
			if (!Bit_Test(Clavier_Pec,6))
			{
				Pos_Pec1=false;
				//OUTPUT_LOW(Led_A_Plus);
			}
		}
    	}
        else
        {
                NUM_STEP_M_A-=Resolution_M;
                NUM_STEP_M_B-=Resolution_M;
                #asm                            // Soustraction en 32 bits
                        MOVF   Resolution_M,W   //NUM_POSI_L -= Resolution_M;
                      	SUBWF  NUM_POSI_L,F
                        BTFSC  03,0
                        GOTO   SUITE2
                        MOVLW  1
                        SUBWF  (&NUM_POSI_L+1),F
                        BTFSC  03,0
                        GOTO   SUITE2
                        SUBWF  (&NUM_POSI_L+2),F
                        BTFSS  03,0
                        DECF   (&NUM_POSI_L+3),F
            	SUITE2:
                #endasm

		Clavier_Pec=PORTB;
		if (Pos_Pec1)
		{
			if (!Bit_Test(Clavier_Pec,6))
			{
				Pos_Pec1 = false;
				//OUTPUT_LOW(Led_A_Plus);
				Pec1_NUM_POSI_L = NUM_POSI_L;
				Pec1_NUM_POSI_H = NUM_POSI_H;
			}
		}
		else
		{
			if (Bit_Test(Clavier_Pec,6))
			{
				Pos_Pec1=true;
				//OUTPUT_HIGH(Led_A_Plus);
			}
		}
     	}

 	step_moteur(NUM_STEP_M_A, NUM_STEP_M_B);
        //CCP1IF = 0;
}

#int_RDA
void IntRDA()
{
 	Buf_serial[Buf_count++]=getc();

	if (Buf_serial[Buf_count-1]==0xA5)
		Sincro++;
	else
		Sincro=0;

	if (Sincro==6)
	{
		Sincro=0;
		Buf_serial[0] = '\0';
                Buf_serial[1] = 0;
                Buf_count = 0;
                Serial_rec = 0;
	}
	else
	{
        	if((Buf_count == 6) && (Buf_serial[0] == Num_Carte))
             		Serial_rec = 1;
        	else if ((Buf_Count >= 6) && (Buf_serial[0] != Num_Carte))
        	{
               		Buf_serial[0] = '\0';
                	Buf_serial[1] = 0;
                	Buf_count = 0;
                	Serial_rec = 0;
        	}
	}
 	//RCIF=0;
 }

//#int_RTCC
void IntRTCC()
{
 	long pente;

 	if (TOIF)
 	{
         	if (Rampe_Plus && !demande_moteur)
         	{
                  	if (Posi)
          		{
    				valide_Posi=true;
    				Posi=false;
   			}
          		if ( V_M > Vitesse_V_M_But )
          		{
           			Pente = V_M >> Accel_M;
           			if (Pente == 0 )
            				Pente = 1;
				Sauve_VM = V_M - pente;
				if (Sauve_VM < V_M)
					V_M -= Pente;
				else V_M=New_V_M;
          		}
          		else
                	{
           			New_V_M_LSB   = Future_V_M_LSB;
                  		New_V_M   = Future_V_M;
                  		New_resolution_M  = Future_resolution_M;
                  		New_Dir_M   = Future_Dir_M;
                  		New_Niveau_Courant_M  = Future_Niveau_Courant_M;
           			if (guidage)
            				Fixe_Ordre_Vitesse_0 = True;
                		demande_Moteur=true;
           			Rampe_Plus = false;

                        	// NUM_COMP *= 2 -> Decalage a gauche sur 32bits

                        	#asm
                             		BCF    03,0
                                	RLF    NUM_COMP_L,F
                                	RLF    (&NUM_COMP_L+1),F
                                	RLF    (&NUM_COMP_L+2),F
                                	RLF    (&NUM_COMP_L+3),F
                        	#endasm
           		}
  		}

        	if (Rampe_Moins && !demande_moteur)
  		{
          		if ( V_M < New_V_M )
   			{
           			Pente = V_M >> Accel_M;
           			if (Pente == 0 )
            				Pente = 1;
				Sauve_VM = V_M + pente;
				if (Sauve_VM > V_M)
					V_M += Pente;
				else V_M=New_V_M;
   			}
          		else
                	{
           			if (!Inverse_Sens)
    				{
            				New_V_M_LSB    = Future_V_M_LSB;
                   			New_V_M   = Future_V_M;
                   			New_resolution_M  = Future_resolution_M;
                   			New_Dir_M   = Future_Dir_M;
                   			New_Niveau_Courant_M  = Future_Niveau_Courant_M;
            				if (guidage)
             					Fixe_Ordre_Vitesse_0 = True;
     					if (Valide_Posi)
      						Valide_Posi=false;
                          		demande_Moteur=true;
            				Rampe_Moins = false;
    				}
           			else
    				{
            				Vitesse_V_M_But  = Future_V_M;
            				New_resolution_M = Future_resolution_M;  // Enclanche l'inversion de sens + resolution
                   			New_Niveau_Courant_M  = Future_Niveau_Courant_M; // le nouveau niveau de courant
            				New_Dir_M   = Future_Dir_M;   // et la valeur de la vitesse par Future_V_M
                          		demande_Moteur=true;
                          		rampe_plus=true;     // Puis lance la rampe plus!
                          		rampe_moins=false;
            				Inverse_Sens=false;
    				}
           		}
   		}
  		TOIF=0;
    	}
}

/*************************************************************************************/
/******************************** Programme principal ********************************/
/*************************************************************************************/

main()
{

	//BRGH = 1;
	//SPBRG = 129;

        SETUP_ADC_PORTS(NO_ANALOGS);

        SET_TRIS_A(0x00);           // Pins du PORTA en output
        SET_TRIS_B(0xFF);           // Pins du PORTB en input
        SET_TRIS_C(0xD0);           // PORTC.7 (RX) en input, autres Pins du PORTC en output
        SET_TRIS_D(0x00);           // Pins du PORTD en output
        SET_TRIS_E(0x07);           // PORTE en input

	Delay_ms( 500 );

        SETUP_TIMER_1(T1_INTERNAL|T1_DIV_BY_8);
        setup_ccp1(CCP_COMPARE_INT|CCP_COMPARE_RESET_TIMER);    // Configure CCP1
 	//setup_counters(RTCC_INTERNAL,WDT_2304MS);
        //setup_timer_2(T2_DIV_BY_16,255,15);
        setup_counters(RTCC_INTERNAL,RTCC_DIV_256);

	OUTPUT_LOW(PEC3);
        //CCP1IF = 0;
        TOIF = 0;
	//TMR2IF = 0;
 	//RCIF = 0;

        PORT_B_PULLUPS(TRUE);
        OUTPUT_LOW(PHASE);  // inicialise le MAX505
        OUTPUT_HIGH(WR_N);
        OUTPUT_HIGH(LDAC_N);
 	OUTPUT_HIGH(Led_A_Plus);
 	OUTPUT_HIGH(Led_A_Moins);

 	Selec_Vit_Ram=false;
 	Vit_Ram=0xFFFF;
 	Vit_Ram_LSB=0;
 	resol_Ram=0;
 	Dir_Ram=0;

        demande_Moteur=true;
        Rampe_plus=false;
        Rampe_moins=false;
        Change = false;
        Guidage = false;
	//Siderale=true;
        Fixe_Ordre_Vitesse_0 = false;
        Inverse_Sens = 0;

 	init_ext_eeprom();
        //init_I2C();
 	//Init_Valeur_EEPROM();

        New_V_M_LSB =  READ_EXT_EEPROM(VM_Guidage_LSB);
        New_V_M =   Init_PARAMETRE(VM_Guidage);
        New_resolution_M =  READ_EXT_EEPROM(Resol_Guidage);
        New_Dir_M=   READ_EXT_EEPROM(Dir_Guidage);
 	//Ancien_Dir_M = New_Dir_M;
 	Dir_M = 0;          // Important, sinon le moteur peut ne pas avoir le bon sens au depart!!!
        New_Niveau_Courant_M =  READ_EXT_EEPROM(Courant_Guidage);
        Accel_M =   READ_EXT_EEPROM(VM_acce);
 	Sens =    READ_EXT_EEPROM(Sens_Raq_Led);

 	if (READ_EXT_EEPROM(Resol_Guidage)!=0)   // Etabli une resolution Mini pour les calculs si resolution=0;
  		resol_Min = READ_EXT_EEPROM(Resol_Guidage);
 	else if (READ_EXT_EEPROM(Resol_Corec_Plus)!=0)
  		resol_Min = READ_EXT_EEPROM(Resol_Corec_Plus);
 	else resol_Min = READ_EXT_EEPROM(Resol_Corec_Moins);


        NUM_STEP_M_A=64;
        NUM_STEP_M_B=192;
 	OUTPUT_HIGH(DIR_PHASEA);
        Flag_DIR_PHASEA = 1;
        OUTPUT_HIGH(DIR_PHASEB);
        Flag_DIR_PHASEB = 1;

        Ordre_Vitesse=0;

        step_moteur(NUM_STEP_M_A, NUM_STEP_M_B);

        Posi = false;
        Buf_serial[0] = '\0';
        Buf_serial[1] = 0;
        Buf_count = 0;
        Serial_rec = false;
 	Raq_rec = false;
	Sincro = 0;
 	NUM_POSI_H = 0x4000;
 	NUM_POSI_L = 0x0000;
	Pec1_NUM_POSI_H = 0x4000;
 	Pec1_NUM_POSI_L = 0x0000;
	Pos_Pec1 = 0;


        Num_Carte = PortE & 0x07;
        Num_Carte += 30;

 	Delay_ms( 500 );

        enable_interrupts(int_ccp1);

        enable_interrupts(int_rda);
        enable_interrupts(global);

 	OUTPUT_LOW(Led_A_Plus);
 	OUTPUT_LOW(Led_A_Moins);

        while(true)
        {
        	//restart_wdt();

         	if(CLAVIER!=(PORTB & 0x3F)) // Verifie si ST4 ou Raquette modifie.
         	{
   			Clavier=PORTB & 0x3F;
          		raquette();
         	}

		//restart_wdt();

                if(Serial_rec || Raq_rec)
                 	traite_serial();

		//restart_wdt();

         	if(change)
          		Etat_Moteur();

		//restart_wdt();

 		 if(Rampe_Plus || Rampe_Moins)
   			IntRTCC();
 	}
}
